@@ -1,15 +1,20 @@
-# Copyrights 2001-2017 by [Mark Overmeer].
+# Copyrights 2001-2019 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.02.
-use strict;
+# This code is part of distribution Mail-Box-IMAP4.  Meta-POD processed with
+# OODoc into POD and HTML manual-pages.  See README.md
+# Copyright Mark Overmeer.  Licensed under the same terms as Perl itself.
 
 package Mail::Box::IMAP4::Head;
 use vars '$VERSION';
-$VERSION = '3.001';
+$VERSION = '3.004';
 
 use base 'Mail::Message::Head';
 
+use warnings;
+use strict;
+
 use Date::Parse;
 
 
